@@ -23,8 +23,10 @@
 #define _NOTEBASE_HPP_
 
 #include <map>
+#include <optional>
 #include <vector>
 
+#include <glibmm/object.h>
 #include <glibmm/ustring.h>
 #include <sigc++/signal.h>
 
@@ -175,7 +177,7 @@ public:
     {
       return *m_data;
     }
-  virtual const Glib::ustring & text();
+  virtual const Glib::ustring & text() const;
   virtual void set_text(Glib::ustring && t);
 private:
   std::unique_ptr<NoteData> m_data;
@@ -183,13 +185,13 @@ private:
 
 
 class NoteBase
-  : public std::enable_shared_from_this<NoteBase>
-  , public sigc::trackable
+  : public Glib::Object
+  , public std::enable_shared_from_this<NoteBase>
 {
 public:
-  typedef std::shared_ptr<NoteBase> Ptr;
-  typedef std::weak_ptr<NoteBase> WeakPtr;
-  typedef std::vector<Ptr> List;
+  typedef Glib::RefPtr<NoteBase> Ptr;
+  typedef std::reference_wrapper<NoteBase> Ref;
+  typedef std::optional<Ref> ORef;
 
   static Glib::ustring url_from_path(const Glib::ustring &);
   static std::vector<Glib::ustring> parse_tags(const xmlNodePtr tagnodes);
@@ -230,7 +232,7 @@ public:
       return m_file_path;
     }
   Glib::ustring get_complete_note_xml();
-  const Glib::ustring & xml_content()
+  const Glib::ustring & xml_content() const
     {
       return data_synchronizer().text();
     }
@@ -251,15 +253,15 @@ public:
     }
   virtual void enabled(bool is_enabled);
 
-  typedef sigc::signal<void(const NoteBase::Ptr &)> SavedHandler;
+  typedef sigc::signal<void(NoteBase&)> SavedHandler;
   SavedHandler signal_saved;
-  typedef sigc::signal<void(const NoteBase::Ptr&, const Glib::ustring&)> RenamedHandler;
+  typedef sigc::signal<void(const NoteBase&, const Glib::ustring&)> RenamedHandler;
   RenamedHandler signal_renamed;
   typedef sigc::signal<void(const NoteBase&, const Tag::Ptr&)> TagAddedHandler;
   TagAddedHandler signal_tag_added;
   typedef sigc::signal<void(const NoteBase&, const Tag &)> TagRemovingHandler;
   TagRemovingHandler signal_tag_removing;
-  typedef sigc::signal<void(const NoteBase::Ptr&, const Glib::ustring&)> TagRemovedHandler;
+  typedef sigc::signal<void(const NoteBase&, const Glib::ustring&)> TagRemovedHandler;
   TagRemovedHandler signal_tag_removed;
 protected:
   virtual const NoteDataBufferSynchronizerBase & data_synchronizer() const = 0;
