@@ -1,7 +1,7 @@
 /*
  * gnote
  *
- * Copyright (C) 2010-2014,2017,2019-2022 Aurimas Cernius
+ * Copyright (C) 2010-2014,2017,2019-2024 Aurimas Cernius
  * Copyright (C) 2010 Debarshi Ray
  * Copyright (C) 2009 Hubert Figuiere
  *
@@ -43,6 +43,7 @@ namespace gnote {
     , m_notebook_manager(*this)
     , m_addin_mgr(NULL)
     , m_note_archiver(*this)
+    , m_save_timeout(0)
   {
   }
 
@@ -86,7 +87,7 @@ namespace gnote {
     }
 
     m_notebook_manager.init();
-    m_gnote.signal_quit.connect(sigc::mem_fun(*this, &NoteManager::on_exiting_event));
+    gnote().signal_quit.connect(sigc::mem_fun(*this, &NoteManager::on_exiting_event));
   }
 
   NoteManager::~NoteManager()
@@ -96,7 +97,7 @@ namespace gnote {
 
   AddinManager *NoteManager::create_addin_manager()
   {
-    return new AddinManager(m_gnote, *this, m_preferences, IGnote::conf_dir());
+    return new AddinManager(gnote(), *this, m_preferences, IGnote::conf_dir());
   }
 
   void NoteManager::create_start_notes ()
@@ -145,12 +146,12 @@ namespace gnote {
         "</note-content>");
 
     try {
-      NoteBase::Ptr start_note = create (_("Start Here"), std::move(start_note_content));
-      start_note->queue_save (CONTENT_CHANGED);
-      m_preferences.start_note_uri(start_note->uri());
+      auto & start_note = create(_("Start Here"), std::move(start_note_content));
+      start_note.queue_save(CONTENT_CHANGED);
+      m_preferences.start_note_uri(start_note.uri());
 
-      NoteBase::Ptr links_note = create (_("Using Links in Gnote"), std::move(links_note_content));
-      links_note->queue_save (CONTENT_CHANGED);
+      auto & links_note = create(_("Using Links in Gnote"), std::move(links_note_content));
+      links_note.queue_save(CONTENT_CHANGED);
     } 
     catch (const std::exception & e) {
       ERR_OUT(_("Error creating start notes: %s"), e.what());
@@ -163,7 +164,7 @@ namespace gnote {
 
     for(auto & file_path : files) {
       try {
-        Note::Ptr note = Note::load(std::move(file_path), *this, m_gnote);
+        Note::Ptr note = Note::load(std::move(file_path), *this, gnote());
         add_note(note);
       } 
       catch (const std::exception & e) {
@@ -180,9 +181,9 @@ namespace gnote {
     auto start_note_uri = m_preferences.start_note_uri();
     if (start_note_uri.empty() || !find_by_uri(start_note_uri)) {
       // Attempt to find an existing Start Here note
-      NoteBase::Ptr start_note = find (_("Start Here"));
-      if (start_note) {
-        m_preferences.start_note_uri(start_note->uri());
+      auto start_note = find(_("Start Here"));
+      if(start_note) {
+        m_preferences.start_note_uri(start_note.value().get().uri());
       }
     }
   }
@@ -195,11 +196,9 @@ namespace gnote {
     // Load all the addins for our notes.
     // Iterating through copy of notes list, because list may be
     // changed when loading addins.
-    NoteBase::List notesCopy(m_notes);
+    decltype(m_notes) notesCopy(m_notes);
     for(const NoteBase::Ptr & iter : notesCopy) {
-      Note::Ptr note(std::static_pointer_cast<Note>(iter));
-
-      m_addin_mgr->load_addins_for_note (note);
+      m_addin_mgr->load_addins_for_note(*iter);
     }
   }
 
@@ -209,27 +208,19 @@ namespace gnote {
 
     for(auto file : files) {
       const Glib::RefPtr<Gio::File> src = Gio::File::create_for_path(file);
-      const Glib::ustring dest_path
-          = Glib::build_filename(notes_dir(),
-                                 Glib::path_get_basename(file));
-      const Glib::RefPtr<Gio::File> dest = Gio::File::create_for_path(
-                                                        dest_path);
-      src->copy(dest, Gio::FILE_COPY_NONE);
+      const Glib::ustring dest_path = Glib::build_filename(notes_dir(), Glib::path_get_basename(file.c_str()));
+      const Glib::RefPtr<Gio::File> dest = Gio::File::create_for_path(dest_path);
+      src->copy(dest, Gio::File::CopyFlags::NONE);
     }
 
-    const Glib::ustring old_backup_dir = Glib::build_filename(
-                                         old_note_dir,
-                                         "Backup");
+    const Glib::ustring old_backup_dir = Glib::build_filename(old_note_dir, "Backup");
     files = sharp::directory_get_files_with_ext(old_backup_dir, ".note");
 
     for(auto file : files) {
       const Glib::RefPtr<Gio::File> src = Gio::File::create_for_path(file);
-      const Glib::ustring dest_path
-          = Glib::build_filename(m_backup_dir,
-                                 Glib::path_get_basename(file));
-      const Glib::RefPtr<Gio::File> dest = Gio::File::create_for_path(
-                                                        dest_path);
-      src->copy(dest, Gio::FILE_COPY_NONE);
+      const Glib::ustring dest_path = Glib::build_filename(m_backup_dir, Glib::path_get_basename(file.c_str()));
+      const Glib::RefPtr<Gio::File> dest = Gio::File::create_for_path(dest_path);
+      src->copy(dest, Gio::File::CopyFlags::NONE);
     }
   }
 
@@ -241,7 +232,7 @@ namespace gnote {
       
     // Use a copy of the notes to prevent bug #510442 (crash on exit
     // when iterating the notes to save them.
-    NoteBase::List notesCopy(m_notes);
+    decltype(m_notes) notesCopy(m_notes);
     for(const NoteBase::Ptr & note : notesCopy) {
       note->save();
     }
@@ -249,66 +240,62 @@ namespace gnote {
 
   NoteBase::Ptr NoteManager::note_load(Glib::ustring && file_name)
   {
-    return Note::load(std::move(file_name), *this, m_gnote);
+    return Note::load(std::move(file_name), *this, gnote());
   }
 
 
-  NoteBase::Ptr NoteManager::create_note(Glib::ustring && title, Glib::ustring && body, Glib::ustring && guid)
+  Note & NoteManager::create_note(Glib::ustring && title, Glib::ustring && body, Glib::ustring && guid)
   {
     bool select_body = body.empty();
-    auto new_note = NoteManagerBase::create_note(std::move(title), std::move(body), std::move(guid));
+    auto & new_note = static_cast<Note&>(NoteManagerBase::create_note(std::move(title), std::move(body), std::move(guid)));
     if(select_body) {
       // Select the inital text so typing will overwrite the body text
-      std::static_pointer_cast<Note>(new_note)->get_buffer()->select_note_body();
+      new_note.get_buffer()->select_note_body();
     }
     return new_note;
   }
 
   // Create a new note with the specified Xml content
-  NoteBase::Ptr NoteManager::create_new_note(Glib::ustring && title, Glib::ustring && xml_content, Glib::ustring && guid)
+  Note & NoteManager::create_new_note(Glib::ustring && title, Glib::ustring && xml_content, Glib::ustring && guid)
   {
-    NoteBase::Ptr new_note = NoteManagerBase::create_new_note(std::move(title), std::move(xml_content), std::move(guid));
+    auto & new_note = static_cast<Note&>(NoteManagerBase::create_new_note(std::move(title), std::move(xml_content), std::move(guid)));
 
     // Load all the addins for the new note
-    m_addin_mgr->load_addins_for_note(std::static_pointer_cast<Note>(new_note));
+    m_addin_mgr->load_addins_for_note(new_note);
 
     return new_note;
   }
 
   NoteBase::Ptr NoteManager::note_create_new(Glib::ustring && title, Glib::ustring && file_name)
   {
-    return Note::create_new_note(std::move(title), std::move(file_name), *this, m_gnote);
+    return Note::create_new_note(std::move(title), std::move(file_name), *this, gnote());
   }
 
-  NoteBase::Ptr NoteManager::get_or_create_template_note()
+  Note & NoteManager::get_or_create_template_note()
   {
-    NoteBase::Ptr template_note = NoteManagerBase::get_or_create_template_note();
+    Note & note = static_cast<Note&>(NoteManagerBase::get_or_create_template_note());
 
     // Select the initial text
-    Glib::RefPtr<NoteBuffer> buffer = std::static_pointer_cast<Note>(template_note)->get_buffer();
+    Glib::RefPtr<NoteBuffer> buffer = note.get_buffer();
     buffer->select_note_body();
-
-    return template_note;
+    return note;
   }
 
   // Creates a new note with the given title and guid with body based on
   // the template note.
-  NoteBase::Ptr NoteManager::create_note_from_template(Glib::ustring && title, const NoteBase::Ptr & template_note, Glib::ustring && guid)
+  Note & NoteManager::create_note_from_template(Glib::ustring && title, const NoteBase & template_note, Glib::ustring && guid)
   {
     auto title_size = title.size();
-    NoteBase::Ptr new_note = NoteManagerBase::create_note_from_template(std::move(title), template_note, std::move(guid));
-    if(new_note == 0) {
-      return new_note;
-    }
+    auto & new_note = static_cast<Note&>(NoteManagerBase::create_note_from_template(std::move(title), template_note, std::move(guid)));
 
     // Copy template note's properties
-    Glib::RefPtr<Gtk::TextBuffer> buffer = std::static_pointer_cast<Note>(new_note)->get_buffer();
+    Glib::RefPtr<Gtk::TextBuffer> buffer = new_note.get_buffer();
     Gtk::TextIter cursor, selection;
     Tag::Ptr template_save_selection = m_tag_manager.get_or_create_system_tag(ITagManager::TEMPLATE_NOTE_SAVE_SELECTION_SYSTEM_TAG);
-    if(template_note->contains_tag(template_save_selection)) {
-      Glib::ustring template_title = template_note->get_title();
-      int cursor_pos = template_note->data().cursor_position();
-      int selection_bound = template_note->data().selection_bound_position();
+    if(template_note.contains_tag(template_save_selection)) {
+      Glib::ustring template_title = template_note.get_title();
+      int cursor_pos = template_note.data().cursor_position();
+      int selection_bound = template_note.data().selection_bound_position();
       if(cursor_pos == 0) {
         // the created note has different title from template, take that into account
         // selection starts at the beginning of title
@@ -350,4 +337,57 @@ namespace gnote {
     return new_note;
   }
 
+  void NoteManager::queue_save(NoteBase & note)
+  {
+    const auto & uri = note.uri();
+    for(const auto & to_save : m_queued_saves) {
+      if(to_save == uri) {
+        return;
+      }
+    }
+
+    m_queued_saves.push_back(uri);
+    if(m_save_timeout) {
+      return;
+    }
+
+    auto save_callback = [](gpointer data) -> gboolean {
+      auto & manager = *static_cast<NoteManager*>(data);
+      manager.save_notes();
+      if(manager.m_queued_saves.size()) {
+        return TRUE;
+      }
+
+      manager.m_save_timeout = 0;
+      return FALSE;
+    };
+    m_save_timeout = g_timeout_add_seconds(4, save_callback, this);
+  }
+
+  void NoteManager::save_notes()
+  {
+    if(m_queued_saves.size() == 0) {
+      return;
+    }
+
+    // make copy, just incase saving causes new queues
+    decltype(m_queued_saves) to_save;
+    std::swap(m_queued_saves, to_save);
+
+    for(const auto & uri : to_save) {
+      bool did_find = find_by_uri(uri, [](NoteBase & note) {
+        try {
+          note.save();
+        }
+        catch(const sharp::Exception &e) {
+          ERR_OUT(_("Error while saving: %s"), e.what());
+        }
+      });
+
+      if(!did_find) {
+        ERR_OUT(_("Did not find note with uri '%s', cannot save"), uri.c_str());
+      }
+    }
+  }
+
 }
