@@ -1,7 +1,7 @@
 /*
  * gnote
  *
- * Copyright (C) 2010-2015,2017,2019-2020 Aurimas Cernius
+ * Copyright (C) 2010-2015,2017,2019-2020,2022-2023 Aurimas Cernius
  * Copyright (C) 2009 Hubert Figuiere
  *
  * This program is free software: you can redistribute it and/or modify
@@ -69,7 +69,7 @@ namespace gnote {
   private:
     Gtk::TextIter get_title_end() const;
     Gtk::TextIter get_title_start() const;
-    bool on_editor_focus_out(GdkEventFocus *);
+    void on_editor_focus_out();
     void on_mark_set(const Gtk::TextIter &, const Glib::RefPtr<Gtk::TextMark>&);
     void on_insert_text(const Gtk::TextIter &, const Glib::ustring &, int);
     void on_delete_range(const Gtk::TextIter &,const Gtk::TextIter &);
@@ -165,14 +165,8 @@ namespace gnote {
                       const Gtk::TextIter & start, const Gtk::TextIter &end);
     void on_delete_range(const Gtk::TextIter &,const Gtk::TextIter &);
     void on_insert_text(const Gtk::TextIter &, const Glib::ustring &, int);
-    bool on_button_press(GdkEventButton *);
-    void on_populate_popup(Gtk::Menu *);
-    bool on_popup_menu();
-    void copy_link_activate();
-    void open_link_activate();
 
     NoteTag::Ptr                m_url_tag;
-    Glib::RefPtr<Gtk::TextMark> m_click_mark;
     Glib::RefPtr<Glib::Regex>   m_regex;
     static const char * URL_REGEX;
     static bool  s_text_event_connected;
@@ -184,20 +178,20 @@ namespace gnote {
   {
   public:
     static ApplicationAddin *create();
-    static void highlight_in_block(NoteManagerBase &, const Note::Ptr &, const Gtk::TextIter &, const Gtk::TextIter &);
-    static void do_highlight(NoteManagerBase &, const Note::Ptr &, const TrieHit<NoteBase::WeakPtr> &, const Gtk::TextIter & ,const Gtk::TextIter &);
-    static void remove_link_tag(const Note::Ptr & note, const Glib::RefPtr<Gtk::TextTag> & tag, const Gtk::TextIter & start, const Gtk::TextIter & end);
+    static void highlight_in_block(NoteManagerBase &, Note &, const Gtk::TextIter &, const Gtk::TextIter &);
+    static void do_highlight(NoteManagerBase &, Note &, const TrieHit<Glib::ustring> &, const Gtk::TextIter & ,const Gtk::TextIter &);
+    static void remove_link_tag(Note & note, const Glib::RefPtr<Gtk::TextTag> & tag, const Gtk::TextIter & start, const Gtk::TextIter & end);
 
     AppLinkWatcher();
     virtual void initialize() override;
     virtual void shutdown() override;
     virtual bool initialized() override;
   private:
-    static bool contains_text(const NoteBase::Ptr & note, const Glib::ustring & text);
-    static void highlight_note_in_block(NoteManagerBase &, const Note::Ptr &, const NoteBase::Ptr &, const Gtk::TextIter &, const Gtk::TextIter &);
-    void on_note_added(const NoteBase::Ptr &);
-    void on_note_deleted(const NoteBase::Ptr &);
-    void on_note_renamed(const NoteBase::Ptr&, const Glib::ustring&);
+    static bool contains_text(const NoteBase & note, const Glib::ustring & text);
+    static void highlight_note_in_block(NoteManagerBase &, Note &, const NoteBase &, const Gtk::TextIter &, const Gtk::TextIter &);
+    void on_note_added(NoteBase &);
+    void on_note_deleted(NoteBase &);
+    void on_note_renamed(const NoteBase&, const Glib::ustring&);
 
     bool m_initialized;
     sigc::connection m_on_note_deleted_cid;
@@ -216,7 +210,6 @@ namespace gnote {
     virtual void on_note_opened() override;
 
   private:
-    void do_highlight(const TrieHit<NoteBase::WeakPtr> & , const Gtk::TextIter &,const Gtk::TextIter &);
     void highlight_in_block(const Gtk::TextIter &,const Gtk::TextIter &);
     void unhighlight_in_block(const Gtk::TextIter &,const Gtk::TextIter &);
     void on_delete_range(const Gtk::TextIter &,const Gtk::TextIter &);
@@ -278,9 +271,9 @@ namespace gnote {
       }
   private:
     void _init_static();
-    bool on_editor_key_press(GdkEventKey*);
-    bool on_editor_key_release(GdkEventKey*);
-    bool on_editor_motion(GdkEventMotion *);
+    bool on_editor_key_press(guint, guint, Gdk::ModifierType);
+    void on_editor_motion(double, double);
+    void on_button_release(double, double, Gdk::ModifierType);
     bool m_hovering_on_link;
     static bool s_static_inited;
     static Glib::RefPtr<Gdk::Cursor> s_normal_cursor;
@@ -301,7 +294,7 @@ namespace gnote {
   private:
     void on_tag_added(const NoteBase&, const Tag::Ptr&);
     void on_tag_removing(const NoteBase&, const Tag &);
-    void on_tag_removed(const NoteBase::Ptr&, const Glib::ustring&);
+    void on_tag_removed(const NoteBase&, const Glib::ustring&);
 
     sigc::connection m_on_tag_added_cid;
     sigc::connection m_on_tag_removing_cid;
